# Variables y funciones comunes a varios scripts de Auto Arte
# Igalia, S.L.
# $Id: common.pm,v 1.4 2003/07/10 19:19:37 jmcasanova Exp $

package common;

use Fcntl;
use POSIX qw(strftime);

BEGIN {
	use Exporter ();
	@ISA = qw(Exporter);
	@EXPORT = qw($servidor $cmd_servidor $dir_trabajo $dir_tmp_log
                     $dir_proceso_logs $dir_proceso_todo $dir_logs
                     $dir_spool $dir_todo $dir_debian $dir_fotos 
                     $dir_log_sist @dirs_necesarios $EXIT_NODIR
                     $EXIT_OK $EXIT_MANYLOGS $EXIT_BADMD5 $EXIT_PROCTODO
                     $EXIT_ERRBZ2 $EXIT_NOCONNECTION $EXIT_ERRSCP
                     $EXIT_MANYTODOS $EXIT_GENTODO &logmsg &getmd5
                     &basename &crear_lock &borrar_lock &shellescape
                     &save_file &panic_mail &delete_and_append);
}

# La IP del servidor
our $servidor = "A.B.C.D";

# Script localizado en el servidor, que ejecutan los clientes en remoto
our $cmd_servidor = "/usr/lib/fisterra/servidor.pl";

# Directorio de trabajo, donde se almacenan los ficheros de log mientras
# se trabaja con ellos, etc.
# Debera existir siempre, y debera borrarse todo su contenido (menos
# los directorios) al reiniciar la mquina (Debian lo hace).
our $dir_trabajo = "/var/run/fisterra";

# Directorio del servidor en el que se copian inicialmente
# los ficheros de log de los clientes, antes de ser comprobados
# Se debe crear en cada arranque de la mquina
our $dir_tmp_log = "$dir_trabajo/incoming";

# Directorio de trabajo, en el que estn los ficheros con los logs y se
# generan los ficheros TODO. Se borra al reiniciar la mquina.
our $dir_proceso_logs = "$dir_trabajo/logs";
our $dir_proceso_todo = "$dir_trabajo/todo";

# Directorio de los logs y de los TODOs
our $dir_spool = "/var/spool/fisterra";
our $dir_logs = "$dir_spool/logs";
our $dir_todo = "$dir_spool/todo";
our $dir_debian = "$dir_spool/debian";
our $dir_fotos = "$dir_spool/fotos";

# Directorio donde se guardan los logs antiguos
our $dir_log_sist = "/var/log/fisterra";
my $fichero_log_sist = "$dir_log_sist/sincronizacionbd.log";

# Directorios que han de existir para que todo funcione bien
our @dirs_necesarios = (
	$dir_trabajo,
	$dir_tmp_log,
	$dir_proceso_logs,
	$dir_proceso_todo,
	$dir_spool,
	$dir_logs,
	$dir_todo,
	$dir_debian,
	$dir_fotos,
	$dir_log_sist
);

# Cdigos de salida
our $EXIT_OK = 0;
our $EXIT_MANYLOGS = 1; # Hay ms de un fichero de log en el directorio
our $EXIT_BADMD5 = 2; # El MD5 de un fichero es incorrecto
our $EXIT_NOCONNECTION = 3; # La conexin se ha interrumpido
our $EXIT_ERRSCP = 4; # Se ha producido un error al copiar los TODOs
our $EXIT_MANYTODOS = 5; # Hay ms de un fichero de todo en el directorio
our $EXIT_ERRBZ2 = 6; # Error al comprimir con bzip2
our $EXIT_GENTODO = 7; # Error al generar los TODOs
our $EXIT_NODIR = 8; # Falta algn directorio necesario
our $EXIT_PROCTODO = 9; # Error al procesar el TODO

########################################
# FUNCIONES VARIAS                     #
# Esta zona ya no tiene mas variables, #
# no hace falta tocar nada desde aqui  #
########################################

my $fichero_lock_log = "$dir_trabajo/systemlog.lock";
my $fichero_lock_gentodo = "$dir_trabajo/gentodo.lock";

# Esta funcin crea un fichero de lock. Si no se especifica cual se pasa
# $fichero_lock_gentodo
sub crear_lock {
	my $lockfile = shift;
	$lockfile = $fichero_lock_gentodo if (!defined($lockfile));
	while (!sysopen(LOCK, $lockfile, O_RDWR|O_CREAT|O_EXCL)) {
		sleep 1;
	}
	close LOCK;
}

# Esta funcin elimina el fichero de lock
sub borrar_lock {
	my $lockfile = shift;
	$lockfile = $fichero_lock_gentodo if (!defined($lockfile));
	unlink $lockfile;
}

# Esta funcin recibe como parmetro el nombre del proceso y una lnea,
# que se enva al fichero de log, precedida por el PID del proceso
sub logmsg {
	my $name = shift;
	my $l = shift;
	crear_lock($fichero_lock_log);
	open (FICH, ">>$fichero_log_sist");
	my $fecha = strftime "%a %d %b %H:%M:%S %Y", localtime;
	print FICH "$fecha: $name\[$$\] $l\n";
	close (FICH);
	borrar_lock($fichero_lock_log);
}

# Esta funcin calcula el MD5 de un fichero
sub getmd5 {
	my $file = shift;
	$file = shellescape($file);
	my $md5sum = `md5sum $file`;
	$md5sum =~ s/\s.*//;
	chomp($md5sum);
	return $md5sum;
}

# Esta funcin devuelve el nombre de un fichero quitando el directorio
sub basename {
	my $name = shift;
	$name =~ s/^.*\///;
	return $name;
}

# Esta funcion escapa los caracteres especiales del shell
sub shellescape {
	my $cadena = shift;
	$cadena =~ s/([|&;\(\)<> ])/\\$1/g;
	return $cadena
}

# Esta funcin mueve el fichero al directorio de logs, indicando la fecha
sub save_file {
	my $fichero = shift;
	my $hora = strftime "%Y%m%d%H%M%S", gmtime;
	system("mv","-f",$fichero,
                   ("$dir_log_sist/$hora-" . basename($fichero)));
}

# Enva un mail a una direccin del mantenedor con el log del sistema
sub panic_mail {
	my $subject = "\"Error durante la replicacion\"";
	my $email = "usuario\@servidor";
	system("mail -s $subject $email < $fichero_log_sist");
}

# Concatena un fichero al final de otro, borrando el original
sub delete_and_append {
	my $origen = shift;
	my $destino = shift;
	open (ORIGEN, $origen);
	open (DESTINO, ">>$destino");
	print DESTINO <ORIGEN>;
	close(DESTINO);
	close(ORIGEN);
	unlink($origen);
}
